@@ -37,6 +37,8 @@
 #include <bluetooth/hci.h>
 
 #include "src/shared/util.h"
+#include "src/shared/crypto.h"
+#include "src/shared/ecc.h"
 #include "monitor/mainloop.h"
 #include "monitor/bt.h"
 
@@ -47,6 +49,7 @@
 struct bt_le {
 	volatile int ref_count;
 	int vhci_fd;
+	struct bt_crypto *crypto;
 
 	uint8_t  event_mask[16];
 	uint16_t manufacturer;
@@ -76,6 +79,8 @@ struct bt_le {
 
 	uint8_t  le_white_list_size;
 	uint8_t  le_states[8];
+
+	uint8_t  le_local_sk256[32];
 };
 
 static void reset_defaults(struct bt_le *hci)
@@ -90,6 +95,7 @@ static void reset_defaults(struct bt_le *hci)
 	hci->event_mask[2] |= 0x04;	/* Number of Completed Packets */
 	hci->event_mask[3] |= 0x02;	/* Data Buffer Overflow */
 	hci->event_mask[5] |= 0x80;	/* Encryption Key Refresh Complete */
+	//hci->event_mask[7] |= 0x20;	/* LE Meta Event */
 
 	hci->manufacturer = 0x003f;	/* Bluetooth SIG (63) */
 
@@ -105,6 +111,7 @@ static void reset_defaults(struct bt_le *hci)
 	hci->commands[14] |= 0x80;	/* Read Buffer Size */
 	hci->commands[15] |= 0x02;	/* Read BD ADDR */
 	//hci->commands[15] |= 0x20;	/* Read RSSI */
+	hci->commands[22] |= 0x04;	/* Set Event Mask Page 2 */
 	hci->commands[25] |= 0x01;	/* LE Set Event Mask */
 	hci->commands[25] |= 0x02;	/* LE Read Buffer Size */
 	hci->commands[25] |= 0x04;	/* LE Read Local Supported Features */
@@ -119,22 +126,38 @@ static void reset_defaults(struct bt_le *hci)
 	//hci->commands[26] |= 0x10;	/* LE Create Connection */
 	//hci->commands[26] |= 0x20;	/* LE Create Connection Cancel */
 	hci->commands[26] |= 0x40;	/* LE Read White List Size */
-	//hci->commands[26] |= 0x80;	/* LE Clear White List */
+	hci->commands[26] |= 0x80;	/* LE Clear White List */
 	//hci->commands[27] |= 0x01;	/* LE Add Device To White List */
 	//hci->commands[27] |= 0x02;	/* LE Remove Device From White List */
 	//hci->commands[27] |= 0x04;	/* LE Connection Update */
 	//hci->commands[27] |= 0x08;	/* LE Set Host Channel Classification */
 	//hci->commands[27] |= 0x10;	/* LE Read Channel Map */
 	//hci->commands[27] |= 0x20;	/* LE Read Remote Used Features */
-	//hci->commands[27] |= 0x40;	/* LE Encrypt */
-	//hci->commands[27] |= 0x80;	/* LE Rand */
+	hci->commands[27] |= 0x40;	/* LE Encrypt */
+	hci->commands[27] |= 0x80;	/* LE Rand */
 	//hci->commands[28] |= 0x01;	/* LE Start Encryption */
 	//hci->commands[28] |= 0x02;	/* LE Long Term Key Request Reply */
-	//hci->commands[28] |= 0x04;	/* LE Long Term Key Neg Request Reply */
+	//hci->commands[28] |= 0x04;	/* LE Long Term Key Request Negative Reply */
 	hci->commands[28] |= 0x08;	/* LE Read Supported States */
 	//hci->commands[28] |= 0x10;	/* LE Receiver Test */
 	//hci->commands[28] |= 0x20;	/* LE Transmitter Test */
 	//hci->commands[28] |= 0x40;	/* LE Test End */
+	//hci->commands[33] |= 0x10;	/* LE Remote Connection Parameter Request Reply */
+	//hci->commands[33] |= 0x20;	/* LE Remote Connection Parameter Request Negative Reply */
+	//hci->commands[33] |= 0x40;	/* LE Set Data Length */
+	//hci->commands[33] |= 0x80;	/* LE Read Suggested Default Data Length */
+	//hci->commands[34] |= 0x01;	/* LE Write Suggested Default Data Length */
+	hci->commands[34] |= 0x02;	/* LE Read Local P-256 Public Key */
+	hci->commands[34] |= 0x04;	/* LE Generate DHKey */
+	//hci->commands[34] |= 0x08;	/* LE Add Device To Resolving List */
+	//hci->commands[34] |= 0x10;	/* LE Remove Device From Resolving List */
+	//hci->commands[34] |= 0x20;	/* LE Clear Resolving List */
+	//hci->commands[34] |= 0x40;	/* LE Read Resolving List Size */
+	//hci->commands[34] |= 0x80;	/* LE Read Peer Resolvable Address */
+	//hci->commands[35] |= 0x01;	/* LE Read Local Resolvable Address */
+	//hci->commands[35] |= 0x02;	/* LE Set Address Resolution Enable */
+	//hci->commands[35] |= 0x04;	/* LE Set Resolvable Private Address Timeout */
+	//hci->commands[35] |= 0x08;	/* LE Read Maximum Data Length */
 
 	memset(hci->features, 0, sizeof(hci->features));
 	hci->features[4] |= 0x20;	/* BR/EDR Not Supported */
@@ -143,13 +166,30 @@ static void reset_defaults(struct bt_le *hci)
 	memset(hci->bdaddr, 0, sizeof(hci->bdaddr));
 
 	memset(hci->le_event_mask, 0, sizeof(hci->le_event_mask));
-	hci->le_event_mask[0] = 0x1f;
+	hci->le_event_mask[0] |= 0x01;	/* LE Connection Complete */
+	hci->le_event_mask[0] |= 0x02;	/* LE Advertising Report */
+	hci->le_event_mask[0] |= 0x04;	/* LE Connection Update Complete */
+	hci->le_event_mask[0] |= 0x08;	/* LE Read Remote Used Features Complete */
+	hci->le_event_mask[0] |= 0x10;	/* LE Long Term Key Request */
+	//hci->le_event_mask[0] |= 0x20;	/* LE Remote Connection Parameter Request */
+	//hci->le_event_mask[0] |= 0x40;	/* LE Data Length Change */
+	//hci->le_event_mask[0] |= 0x80;	/* LE Read Local P-256 Public Key Complete */
+	//hci->le_event_mask[1] |= 0x01;	/* LE Generate DHKey Complete */
+	//hci->le_event_mask[1] |= 0x02;	/* LE Enhanced Connection Complete */
+	//hci->le_event_mask[1] |= 0x04;	/* LE Direct Advertising Report */
 
 	hci->le_mtu = 64;
 	hci->le_max_pkt = 1;
 
 	memset(hci->le_features, 0, sizeof(hci->le_features));
 	hci->le_features[0] |= 0x01;	/* LE Encryption */
+	//hci->le_features[0] |= 0x02;	/* Connection Parameter Request Procedure */
+	//hci->le_features[0] |= 0x04;	/* Extended Reject Indication */
+	//hci->le_features[0] |= 0x08;	/* Slave-initiated Features Exchange */
+	hci->le_features[0] |= 0x10;	/* LE Ping */
+	//hci->le_features[0] |= 0x20;	/* LE Data Packet Length Extension */
+	//hci->le_features[0] |= 0x40;	/* LL Privacy */
+	hci->le_features[0] |= 0x80;	/* Extended Scanner Filter Policies */
 
 	memset(hci->le_random_addr, 0, sizeof(hci->le_random_addr));
 
@@ -183,6 +223,8 @@ static void reset_defaults(struct bt_le *hci)
 	hci->le_states[0] |= 0x20;	/* Active Scanning */
 	hci->le_states[0] |= 0x40;	/* Initiating */
 	hci->le_states[0] |= 0x80;	/* Connection */
+
+	memset(hci->le_local_sk256, 0, sizeof(hci->le_local_sk256));
 }
 
 static void send_event(struct bt_le *hci, uint8_t event,
@@ -243,6 +285,26 @@ static void cmd_status(struct bt_le *hci, uint8_t status, uint16_t opcode)
 	send_event(hci, BT_HCI_EVT_CMD_STATUS, &cs, sizeof(cs));
 }
 
+static void le_meta_event(struct bt_le *hci, uint8_t event,
+						void *data, uint8_t len)
+{
+	void *pkt_data;
+
+	if (!(hci->event_mask[7] & 0x20))
+		return;
+
+	pkt_data = alloca(1 + len);
+	if (!pkt_data)
+		return;
+
+	((uint8_t *) pkt_data)[0] = event;
+
+	if (len > 0)
+		memcpy(pkt_data + 1, data, len);
+
+	send_event(hci, BT_HCI_EVT_LE_META_EVENT, pkt_data, 1 + len);
+}
+
 static void cmd_set_event_mask(struct bt_le *hci,
 						const void *data, uint8_t size)
 {
@@ -271,9 +333,9 @@ static void cmd_read_local_version(struct bt_le *hci,
 	struct bt_hci_rsp_read_local_version rsp;
 
 	rsp.status = BT_HCI_ERR_SUCCESS;
-	rsp.hci_ver = 0x06;
+	rsp.hci_ver = 0x08;
 	rsp.hci_rev = cpu_to_le16(0x0000);
-	rsp.lmp_ver = 0x06;
+	rsp.lmp_ver = 0x08;
 	rsp.manufacturer = cpu_to_le16(hci->manufacturer);
 	rsp.lmp_subver = cpu_to_le16(0x0000);
 
@@ -326,6 +388,19 @@ static void cmd_read_bd_addr(struct bt_le *hci, const void *data, uint8_t size)
 	cmd_complete(hci, BT_HCI_CMD_READ_BD_ADDR, &rsp, sizeof(rsp));
 }
 
+static void cmd_set_event_mask_page2(struct bt_le *hci,
+						const void *data, uint8_t size)
+{
+	const struct bt_hci_cmd_set_event_mask_page2 *cmd = data;
+	uint8_t status;
+
+	memcpy(hci->event_mask + 8, cmd->mask, 8);
+
+	status = BT_HCI_ERR_SUCCESS;
+	cmd_complete(hci, BT_HCI_CMD_SET_EVENT_MASK_PAGE2,
+						&status, sizeof(status));
+}
+
 static void cmd_le_set_event_mask(struct bt_le *hci,
 						const void *data, uint8_t size)
 {
@@ -577,6 +652,49 @@ static void cmd_le_read_white_list_size(struct bt_le *hci,
 							&rsp, sizeof(rsp));
 }
 
+static void cmd_le_clear_white_list(struct bt_le *hci,
+						const void *data, uint8_t size)
+{
+	uint8_t status;
+
+	status = BT_HCI_ERR_SUCCESS;
+	cmd_complete(hci, BT_HCI_CMD_LE_CLEAR_WHITE_LIST,
+						&status, sizeof(status));
+}
+
+static void cmd_le_encrypt(struct bt_le *hci, const void *data, uint8_t size)
+{
+	const struct bt_hci_cmd_le_encrypt *cmd = data;
+	struct bt_hci_rsp_le_encrypt rsp;
+
+	if (!bt_crypto_e(hci->crypto, cmd->key, cmd->plaintext, rsp.data)) {
+		cmd_status(hci, BT_HCI_ERR_COMMAND_DISALLOWED,
+					BT_HCI_CMD_LE_ENCRYPT);
+		return;
+	}
+
+	rsp.status = BT_HCI_ERR_SUCCESS;
+
+	cmd_complete(hci, BT_HCI_CMD_LE_ENCRYPT, &rsp, sizeof(rsp));
+}
+
+static void cmd_le_rand(struct bt_le *hci, const void *data, uint8_t size)
+{
+	struct bt_hci_rsp_le_rand rsp;
+	uint8_t value[8];
+
+	if (!bt_crypto_random_bytes(hci->crypto, value, 8)) {
+		cmd_status(hci, BT_HCI_ERR_COMMAND_DISALLOWED,
+					BT_HCI_CMD_LE_RAND);
+		return;
+	}
+
+	rsp.status = BT_HCI_ERR_SUCCESS;
+	memcpy(&rsp.number, value, 8);
+
+	cmd_complete(hci, BT_HCI_CMD_LE_RAND, &rsp, sizeof(rsp));
+}
+
 static void cmd_le_read_supported_states(struct bt_le *hci,
 						const void *data, uint8_t size)
 {
@@ -589,19 +707,51 @@ static void cmd_le_read_supported_states(struct bt_le *hci,
 							&rsp, sizeof(rsp));
 }
 
+static void cmd_le_read_local_pk256(struct bt_le *hci,
+						const void *data, uint8_t size)
+{
+	struct bt_hci_evt_le_read_local_pk256_complete evt;
+
+	cmd_status(hci, BT_HCI_ERR_SUCCESS, BT_HCI_CMD_LE_READ_LOCAL_PK256);
+
+	evt.status = BT_HCI_ERR_SUCCESS;
+	ecc_make_key(evt.local_pk256, hci->le_local_sk256);
+
+	if (hci->le_event_mask[0] & 0x80)
+		le_meta_event(hci, BT_HCI_EVT_LE_READ_LOCAL_PK256_COMPLETE,
+							&evt, sizeof(evt));
+}
+
+static void cmd_le_generate_dhkey(struct bt_le *hci,
+						const void *data, uint8_t size)
+{
+	const struct bt_hci_cmd_le_generate_dhkey *cmd = data;
+	struct bt_hci_evt_le_generate_dhkey_complete evt;
+
+	cmd_status(hci, BT_HCI_ERR_SUCCESS, BT_HCI_CMD_LE_GENERATE_DHKEY);
+
+	evt.status = BT_HCI_ERR_SUCCESS;
+	ecdh_shared_secret(cmd->remote_pk256, hci->le_local_sk256, evt.dhkey);
+
+	if (hci->le_event_mask[1] & 0x01)
+		le_meta_event(hci, BT_HCI_EVT_LE_GENERATE_DHKEY_COMPLETE,
+							&evt, sizeof(evt));
+}
+
 static const struct {
 	uint16_t opcode;
 	void (*func) (struct bt_le *hci, const void *data, uint8_t size);
 	uint8_t size;
 	bool fixed;
 } cmd_table[] = {
-	{ BT_HCI_CMD_SET_EVENT_MASK,       cmd_set_event_mask,      8, true },
-	{ BT_HCI_CMD_RESET,                cmd_reset,               0, true },
-	{ BT_HCI_CMD_READ_LOCAL_VERSION,   cmd_read_local_version,  0, true },
-	{ BT_HCI_CMD_READ_LOCAL_COMMANDS,  cmd_read_local_commands, 0, true },
-	{ BT_HCI_CMD_READ_LOCAL_FEATURES,  cmd_read_local_features, 0, true },
-	{ BT_HCI_CMD_READ_BUFFER_SIZE,     cmd_read_buffer_size,    0, true },
-	{ BT_HCI_CMD_READ_BD_ADDR,         cmd_read_bd_addr,        0, true },
+	{ BT_HCI_CMD_SET_EVENT_MASK,       cmd_set_event_mask,       8, true },
+	{ BT_HCI_CMD_RESET,                cmd_reset,                0, true },
+	{ BT_HCI_CMD_READ_LOCAL_VERSION,   cmd_read_local_version,   0, true },
+	{ BT_HCI_CMD_READ_LOCAL_COMMANDS,  cmd_read_local_commands,  0, true },
+	{ BT_HCI_CMD_READ_LOCAL_FEATURES,  cmd_read_local_features,  0, true },
+	{ BT_HCI_CMD_READ_BUFFER_SIZE,     cmd_read_buffer_size,     0, true },
+	{ BT_HCI_CMD_READ_BD_ADDR,         cmd_read_bd_addr,         0, true },
+	{ BT_HCI_CMD_SET_EVENT_MASK_PAGE2, cmd_set_event_mask_page2, 8, true },
 
 	{ BT_HCI_CMD_LE_SET_EVENT_MASK,
 				cmd_le_set_event_mask, 8, true },
@@ -624,9 +774,20 @@ static const struct {
 
 	{ BT_HCI_CMD_LE_READ_WHITE_LIST_SIZE,
 				cmd_le_read_white_list_size, 0, true },
+	{ BT_HCI_CMD_LE_CLEAR_WHITE_LIST,
+				cmd_le_clear_white_list, 0, true },
+
+	{ BT_HCI_CMD_LE_ENCRYPT, cmd_le_encrypt, 32, true },
+	{ BT_HCI_CMD_LE_RAND, cmd_le_rand, 0, true },
+
 	{ BT_HCI_CMD_LE_READ_SUPPORTED_STATES,
 				cmd_le_read_supported_states, 0, true },
 
+	{ BT_HCI_CMD_LE_READ_LOCAL_PK256,
+				cmd_le_read_local_pk256, 0, true },
+	{ BT_HCI_CMD_LE_GENERATE_DHKEY,
+				cmd_le_generate_dhkey, 64, true },
+
 	{ }
 };
 
@@ -714,6 +875,8 @@ struct bt_le *bt_le_new(void)
 
 	mainloop_add_fd(hci->vhci_fd, EPOLLIN, vhci_read_callback, hci, NULL);
 
+	hci->crypto = bt_crypto_new();
+
 	return bt_le_ref(hci);
 }
 
@@ -735,6 +898,8 @@ void bt_le_unref(struct bt_le *hci)
 	if (__sync_sub_and_fetch(&hci->ref_count, 1))
 		return;
 
+	bt_crypto_unref(hci->crypto);
+
 	mainloop_remove_fd(hci->vhci_fd);
 
 	close(hci->vhci_fd);
