@@ -158,28 +158,15 @@ static void write_watch_destroy(void *user_data)
 	mgmt->writer_active = false;
 }
 
-static bool can_write_data(struct io *io, void *user_data)
+static bool send_request(struct mgmt *mgmt, struct mgmt_request *request)
 {
-	struct mgmt *mgmt = user_data;
-	struct mgmt_request *request;
 	struct iovec iov;
 	ssize_t ret;
 
-	request = queue_pop_head(mgmt->reply_queue);
-	if (!request) {
-		/* only reply commands can jump the queue */
-		if (!queue_isempty(mgmt->pending_list))
-			return false;
-
-		request = queue_pop_head(mgmt->request_queue);
-		if (!request)
-			return false;
-	}
-
 	iov.iov_base = request->buf;
 	iov.iov_len = request->len;
 
-	ret = io_send(io, &iov, 1);
+	ret = io_send(mgmt->io, &iov, 1);
 	if (ret < 0) {
 		util_debug(mgmt->debug_callback, mgmt->debug_data,
 				"write failed: %s", strerror(-ret));
@@ -187,7 +174,7 @@ static bool can_write_data(struct io *io, void *user_data)
 			request->callback(MGMT_STATUS_FAILED, 0, NULL,
 							request->user_data);
 		destroy_request(request);
-		return true;
+		return false;
 	}
 
 	util_debug(mgmt->debug_callback, mgmt->debug_data,
@@ -199,7 +186,35 @@ static bool can_write_data(struct io *io, void *user_data)
 
 	queue_push_tail(mgmt->pending_list, request);
 
-	return false;
+	return true;
+}
+
+static bool can_write_data(struct io *io, void *user_data)
+{
+	struct mgmt *mgmt = user_data;
+	struct mgmt_request *request;
+	bool can_write;
+
+	request = queue_pop_head(mgmt->reply_queue);
+	if (!request) {
+		/* only reply commands can jump the queue */
+		if (!queue_isempty(mgmt->pending_list))
+			return false;
+
+		request = queue_pop_head(mgmt->request_queue);
+		if (!request)
+			return false;
+
+		can_write = false;
+	} else {
+		/* allow multiple replies to jump the queue */
+		can_write = !queue_isempty(mgmt->reply_queue);
+	}
+
+	if (!send_request(mgmt, request))
+		return true;
+
+	return can_write;
 }
 
 static void wakeup_writer(struct mgmt *mgmt)
@@ -213,6 +228,8 @@ static void wakeup_writer(struct mgmt *mgmt)
 	if (mgmt->writer_active)
 		return;
 
+	mgmt->writer_active = true;
+
 	io_set_write_handler(mgmt->io, can_write_data, mgmt,
 						write_watch_destroy);
 }
@@ -641,6 +658,32 @@ unsigned int mgmt_send(struct mgmt *mgmt, uint16_t opcode, uint16_t index,
 	return request->id;
 }
 
+unsigned int mgmt_send_nowait(struct mgmt *mgmt, uint16_t opcode, uint16_t index,
+				uint16_t length, const void *param,
+				mgmt_request_func_t callback,
+				void *user_data, mgmt_destroy_func_t destroy)
+{
+	struct mgmt_request *request;
+
+	if (!mgmt)
+		return 0;
+
+	request = create_request(opcode, index, length, param,
+					callback, user_data, destroy);
+	if (!request)
+		return 0;
+
+	if (mgmt->next_request_id < 1)
+		mgmt->next_request_id = 1;
+
+	request->id = mgmt->next_request_id++;
+
+	if (!send_request(mgmt, request))
+		return 0;
+
+	return request->id;
+}
+
 unsigned int mgmt_reply(struct mgmt *mgmt, uint16_t opcode, uint16_t index,
 				uint16_t length, const void *param,
 				mgmt_request_func_t callback,
